#if !defined( __MORAY_READER_DEFS )
#define __MORAY_READER_DEFS

/******************* Predefined values ********************/

#define MORAY_PLANAR_IMAGEMAP      0
#define MORAY_SPHERICAL_IMAGEMAP   1
#define MORAY_CYLINDRICAL_IMAGEMAP 2
#define MORAY_TOROIDAL_IMAGEMAP    5

#define MORAY_NULL_TYPE         0
#define MORAY_GROUP_TYPE        1
#define MORAY_CUBE_TYPE         2
#define MORAY_SPHERE_TYPE       3
#define MORAY_CYLINDER_TYPE     4
#define MORAY_CAMERA_TYPE       5
#define MORAY_TSWEEP_TYPE       6
#define MORAY_RSWEEP_TYPE       7
#define MORAY_CSWEEP_TYPE       8
#define MORAY_VOXEL_TYPE        9
#define MORAY_PLIGHT_TYPE      10
#define MORAY_TEXTURE_TYPE     11
#define MORAY_CSG_TYPE         12
#define MORAY_BEZIER_TYPE      13
#define MORAY_BOUND_TYPE       14
#define MORAY_BUMPMAP_TYPE     15
#define MORAY_HEIGHTFIELD_TYPE 16
#define MORAY_CONE_TYPE        17
#define MORAY_TORUS_TYPE       18
#define MORAY_SLIGHT_TYPE      19
#define MORAY_ALIGHT_TYPE      20
#define MORAY_PLANE_TYPE       21
#define MORAY_DISC_TYPE        22

#define MORAY_NO_TEXTURE         1000
#define MORAY_OPAQUE_TEXTURE        0
#define MORAY_CHECKER_TEXTURE       1
#define MORAY_WOOD_TEXTURE          2
#define MORAY_GRADIENT_TEXTURE      9
#define MORAY_IMAGEMAP_TEXTURE     10
#define MORAY_PREDEFINED_TEXTURE   99

#define MORAY_OBJ_NAME_LEN   16
#define MORAY_IMAGE_NAME_LEN 64

#define MORAY_CSG_UNION        0
#define MORAY_CSG_INTERSECTION 1
#define MORAY_CSG_DIFFERENCE   2


/******************* Type Definitions *********************/

typedef char  byte;
typedef short word;
typedef unsigned char  ubyte;
typedef unsigned short uword;
typedef unsigned long  ulong;

typedef struct t_MORAY_MODEL_struct MORAY_Model;
typedef struct t_MORAY_Object_struct MORAY_Object;
typedef struct MRY_Obj_Base_Struct MRY_Base_Data;
typedef struct MRY_Texture_Struct MRY_Texture_Data;
typedef struct MRY_Camera_Struct MRY_Camera_Data;
typedef struct MRY_CSG_Struct MRY_CSG_Data;
typedef struct MRY_Torus_Struct MRY_Torus_Data;
typedef struct MRY_Disc_Struct MRY_Disc_Data;
typedef struct MRY_Heightfield_Struct MRY_Heightfield_Data;
typedef struct MRY_Bezier_Struct MRY_Bezier_Data;
typedef struct MRY_Bezier_Points_Struct MRY_Bezier_Vertices;
typedef struct MRY_Bezier_Patch_Struct MRY_Bezier_Patches;
typedef struct MRY_Sweep_Point_Struct MRY_Sweep_Point;
typedef struct MRY_Sweep_Struct MRY_Sweep_Data;
typedef struct MRY_Point_Light_Struct MRY_Point_Light_Data;
typedef struct MRY_Spot_Light_Struct MRY_Spot_Light_Data;
typedef struct MRY_Area_Light_Struct MRY_Area_Light_Data;
typedef struct MRY_Imagemap_Texture_Struct MRY_Imagemap_Data;
typedef struct MRY_Checker_Texture_Struct MRY_Checker_Data;

struct MRY_CSG_Struct {
   int csg_type;
   int num_objs;
   MORAY_Object *objs;
   };

struct MRY_Torus_Struct {
   double inner_radius, outer_radius;
   };

struct MRY_Disc_Struct {
   double inner_radius, outer_radius;
   };

struct MRY_Heightfield_Struct {
   byte name_len;
   char name[MORAY_IMAGE_NAME_LEN]; /* Name of hf image */
   byte smooth;
   byte boxit;
   word water_level;
   };

struct MRY_Bezier_Points_Struct {
   byte ptype;
   double xpos, ypos, zpos;
   word depnd1, depnd2;
   };

struct MRY_Bezier_Patch_Struct {
   word vindices[16];
   word subdiv;
   };

struct MRY_Bezier_Struct {
   word stop, start;
   double snapx, snapy;
   byte grid;
   double xyvxp, xyvyp;
   double xyvdx, xyvdy;
   double xzvxp, xzvzp;
   double xzvdx, xzvdz;
   double yzvyp, yzvzp;
   double yzvdy, yzvdz;
   int usteps, vsteps;
   double flatn;
   MRY_Bezier_Vertices *points;
   word ptchs, xptch, yptch; /* Total patches, # in x, * in y */
   MRY_Bezier_Patches *patches;
   };

struct MRY_Sweep_Point_Struct {
   double xpos, ypos; /* <x,y> position of node */
   double angl1;      /* Angle from previous node */
   double angl2;      /* Angle to next node */
   };

struct MRY_Sweep_Struct {
   word numseg;          /* Number of nodes in the sweep */
   word subdiv;          /* Subdivision of the segments */
   double viewx, viewy;  /* World coord of top left of editing view */
   double deltax;        /* World width of editing view */
   word expdiv;          /* # of subdivisions when exporting lathe */
   MRY_Sweep_Point *points;
   };

struct MRY_Point_Light_Struct {
   float color[3];
   };

struct MRY_Spot_Light_Struct {
   float color[3];
   float falloff;
   float radius;
   float draw_length;
   word tightness;
   };

struct MRY_Area_Light_Struct {
   float color[3];
   word xres, yres;
   word adaptive;
   byte jittr;
   };

/* General object information - number, name, scale, rotate,
   translate, visibility level, relative visibility flag. */
struct MRY_Obj_Base_Struct {
   word num;          /* Unique object number */
   char name[MORAY_OBJ_NAME_LEN]; /* Name of this object/texture */
   float sx, sy, sz; /* Scale along each axis */
   float rx, ry, rz; /* Rotation components along each axis */
   float tx, ty, tz; /* Translation components along each axis */
   byte vislv, relvs; /* Visibility level, relative visibility flag */
   };

struct MRY_Texture_Struct {
   word texture_type;
   byte name_len;
   char name[MORAY_OBJ_NAME_LEN]; /* Name of the texture */
   float amb;
   float dif; /* Diffuse scale */
   float phg; /* Phong scale */
   float phgx; /* Phong exponent */
   float refl; /* Reflection scale */
   float trns; /* Transparency scale */
   float rfin; /* Index of refraction */
   float cldr; /* Red component of diffuse color */
   float cldg; /* Green component of diffuse color */
   float cldb; /* Blue component of diffuse color */
   float cldf; /* Filter component of diffuse color */
   float bril; /* Brilliance factor */
   float rdnm; /* Random color change scale */
   float spec; /* Specular factor */
   float rghn; /* Roughness factor */
   float turb; /* Turbulence factor */
   float octv; /* Octaves of turbulence */
   float clsr; /* Red component of specular color */
   float clsg; /* Green component of specular color */
   float clsb; /* Blue component of specular color */
   byte metl;  /* Metallic flag */
   void *unique_data;  /* Data unique to this texture type */
   };

struct MRY_Checker_Texture_Struct {
   float cl2r, cl2g, cl2b, cl2f; /* Second color in checker */
   };

struct MRY_Imagemap_Texture_Struct {
   byte name_len;
   char name[MORAY_IMAGE_NAME_LEN]; /* Name of texture file */
   byte smth;
   byte tiled;
   byte mapt;     /* Type of imagemap */
   };

/* Linked list structure for objects within a MORAY file.  The
   allowed values for object types are:
       1 - Group                 2 - Cube
       3 - Sphere                4 - Cylinder
       5 - Camera                6 - Translational Sweep
       7 - Rotational Sweep      8 - Conical Sweep
       9 - Reserved (Voxel)     10 - Point Light
      11 - Texture              12 - CSG
      13 - Bezier_Patch         14 - Bound
      15 - Reserved (BumpMap)   16 - HeightField
      17 - Cone                 18 - Torus
      19 - Spot Light           20 - Area Light
*/

struct MRY_Camera_Struct {
   double eyepx, eyepy, eyepz; /* Location of the camera */
   double eyelx, eyely, eyelz; /* Look-at point */
   double aprtr;               /* FOV in degrees (v or h?) */
   };

struct t_MORAY_Object_struct {
   int type;                /* What kind of object? */
   MRY_Base_Data base_data; /* Common information to all objects */
   MORAY_Object *texture;   /* Texture information (if any) */
   MORAY_Object *bobj;      /* Bounding box information (if any) */
   MORAY_Object *next;      /* Next object in a list */
   byte noshdw;             /* Does this object cast a shadow? */
   void *obj_data;          /* Information unique to this object type */
   };

/* Data type to hold an entire Moray model file */
typedef struct t_MORAY_MODEL_struct {
   int Version;
   int XResolution;
   int Object_Count;
   float Aspect;
   MORAY_Object *Texture_List;
   MORAY_Object *Camera_List;
   MORAY_Object *Object_List;
   };

void error(char *fmt, ...);
void message(char *fmt, ...);
void Read_MORAY_File(FILE *infile, MORAY_Model *model);

#endif /* __MORAY_READER_DEFS */
